import java.util.*;
import java.io.*;

public class CityMap
{
  private HashMap map;

  
  /**
  	initializes an empty city map
  */
  public CityMap()
  {
    map = new HashMap();
  }
  
  /**
  	adds a key value set to the map, the key being a city,
  	the value being the quadrant of the city's location
  	
  	@param city the key to the map
  	@param quad the vlaue associated with the key
  */ 
  public void add(String city, Quadrant quad)
  {
    map.put(city, quad);
  }
  
  /**
  	creates and returns a String representation of the map
  	
  	@return the String representation of this map
  */
  public String toString()
  {
    String str = "Map:\n";
    
    Set keys = map.keySet();   
    Iterator iter = keys.iterator();  
    while (iter.hasNext())
	{
      String city = (String)iter.next();
      Quadrant quad = (Quadrant)map.get(city);
      
      str += "\tCity: " + city + " " + quad.toString() + "\n";
    }
    
    return str;
  }
  
  /**
  	creates and returns a set containing all cities located in
  	the target quadrant.
  	
  	@param target the quadrant to be searched for cities
  	@return the Set that contains all the cities located in the 
  	 specified quadrant
  	*/
  public Set citiesInQuadrant(Quadrant target) {
  
	Set keys = map.keySet();
	Iterator iter = keys.iterator();
	
	Set results = new HashSet();
	while (iter.hasNext()) 
	{
		String city = (String)iter.next();
		if (target.equals((Quadrant)map.get(city)))
		{
			results.add(city);
		}
	}
	return results;
  }
  
  /**
  	builds the map from the given file
  */
  public void buildList() throws FileNotFoundException, IOException
  {
    BufferedReader input = new BufferedReader(new FileReader("Cities.txt"));
    
    String line;
    while ((line = input.readLine()) != null)
    {
      StringTokenizer stok = new StringTokenizer(line);
      while (stok.hasMoreTokens()) {
        String city;
        if (stok.countTokens() > 3)
        {
          city = stok.nextToken() + " " + stok.nextToken();
        } else {
          city = stok.nextToken();
        }
        
        Quadrant quad = new Quadrant(stok.nextToken(), stok.nextToken());
        map.put(city, quad);
      }
    }
  }
  
  /**
  	tests the CityMap
  */
  public static void main(String[] args) throws FileNotFoundException, IOException
  {
    CityMap map = new CityMap();
    map.buildList();
    System.out.println(map);
    Set results = map.citiesInQuadrant(new Quadrant("E", "3"));
	System.out.println("Quadrant E3: " + results);
  }
  
}