import java.util.*;

/**
	file: Quadrant.java
	
	models a map location as a quadrant, specified by a letter
	(for column) and a number (for row). Imagine the map being ordered
	from the upper left corner, so that the first quadrant is "A1".
	
	@author D. Slater
	@version 6/29/2004
*/
public class Quadrant implements Comparable
{
  private String myQuadrant;
  
  /**
  	creates an instance of the Quadrant with the given parameters.
  	
  	@param letter is the column value
  	@param number is the row value
  */
  public Quadrant(String letter, String number)
  {
    myQuadrant = letter + number;
  }
  
  /**
  	creates a copy of the parameter Quadrant
  	
  	@param other the Quadrant to be copied
  */
  public Quadrant(Quadrant other)
  {
    this(other.getQuadrantLetter(), other.getQuadrantNumber());
  }
  
  /**
  	returns this Quadrant
  	
  	@return this Quadrant
  */
  public String getQuadrant() 
  {
    return myQuadrant;
  }
  
  /**
  	returns the letter (column value) of ths quadrant
  	
  	@return the letter (column value) of this Quadrant
  */ 
  public String getQuadrantLetter()
  {
    return myQuadrant.substring(0, 1);
  }
  
  /**
  	returns the number (row value) of ths quadrant
  	
  	@return the number (row value) of this Quadrant
  */ 
  public String getQuadrantNumber()
  {
    return myQuadrant.substring(1);
  }
  
  /**
  	creates and returns a String representation of the quadrant
  	
  	@return the String representation of this Quadrant
  */
  public String toString() 
  {
    return "Quadrant: " + myQuadrant;
  }
  
  /**
  	compares this quadrant to another Object
  	
  	@param other the object to be compared to this Quadrant
  	@return the value 0 if the argement is a Quadrant that is equal
  	 to this Quadrant; a value less than 0 if if the argument is 
  	 a Quadrant that is less than this Quadrant; a value greater than 0 
  	 if the argument is a Quadrant that is greater than this Quadrant
  */
  public int compareTo(Object other)
  {
    return getQuadrant().compareToIgnoreCase(((Quadrant)other).getQuadrant());
  }
  
  
  /**
  	compares this Quadrant to the specified Object
  	
  	@param the other object to be compared to this Quadrant
  	@return true if the Quadrants are equal; false otherwise
  */
  public boolean equals(Object other) 
  {
    if (other instanceof Quadrant) {
      return getQuadrant().equalsIgnoreCase(((Quadrant)other).getQuadrant());
    } else {
      return false;
    }
  }
  
  public int hashCode() {
	  
	 return myQuadrant.hashCode();
  }
  
}