/** 
 * Walker.java
 *
 * Title:	RandomWalker Demo		
 * Description:	This robot has it's own "coin" to randomly generate its
 *		next move. chooses its next step and moves there if the path is not blocked,
 *		as well as how many steps it took.
 *
 * 
 * @author		Slater		
 * @version			
 */


import java.util.*;
import java.awt.Color;


class FishBot extends Fish 
{

    static private Random coin = new Random();
    private int startAve, startStr, numSteps;
	
    /**
    *	constructor initially sets up using the parent c'tor, then initializes
    *	the starting position and number of steps
    */ 
    public FishBot(Environment env, Location loc) 
    {
        // Construct and initialize attributes inherited from fish
        super(env, loc);
        startStr = loc.row();
        startAve = loc.col();
        numSteps = 0;
    }

    /** Constructs a Walker at the specified location and direction in a
    *  given environment.
    *  (Precondition: parameters are non-null; <code>loc</code>
    *  is valid for <code>env</code>.)
    *  @param env    environment in which fish will live
    *  @param loc    location of the new fish in <code>env</code>
    *  @param dir    direction the new fish is facing
    **/
    public FishBot(Environment env, Location loc, Direction dir)
    {
        // Construct and initialize the attributes inherited from Fish.
        super(env, loc, dir);
        startStr = loc.row();
        startAve = loc.col();
        numSteps = 0;
    }
        
    /** Constructs a Walker of the specified color at the specified
    *  location and direction.
    *  (Precondition: parameters are non-null; <code>loc</code> is valid
    *  for <code>env</code>.)
    *  @param env    environment in which fish will live
    *  @param loc    location of the new fish in <code>env</code>
    *  @param dir    direction the new fish is facing
    *  @param col    color of the new fish
    **/
    public FishBot(Environment env, Location loc, Direction dir, Color col)
    {
        // Construct and initialize the attributes inherited from Fish.
        super(env, loc, dir, col);
        startStr = loc.row();
        startAve = loc.col();
        numSteps = 0;
    }

    /**
    * 	The inherited move() method is overwritten,  using nextLOcation(), to allow the 
    *	robot to choose its path, as well as modify its position
    *	and update the number of steps. It will also prevent the robot
    *	from running into an occupied space or out of the environment
    */	
    public void move()
    {

        Location oldLoc = location();
        Location nextLoc = nextLocation();

        if (location() != nextLoc) {
            changeLocation(nextLoc);
            Direction newDir = environment().getDirection(oldLoc, nextLoc);
            changeDirection(newDir);
            numSteps++;
        }
    }

    protected Location nextLocation() {
    
        Location loc = location();
        Location temp;
        
        int flip  = coin.nextInt(4);		// returns possible values 0, 1, 2, or 3
            
        if (flip == 0) {
            temp =  new Location(loc.row(), loc.col() + 1);
        } else if (flip == 1) {			
            temp =  new Location(loc.row() - 1, loc.col());
        } else if (flip == 2) {			
            temp = new Location(loc.row(), loc.col() - 1);
        } else {			
            temp = new Location(loc.row() + 1, loc.col());
        }
        
        if (environment().isEmpty(temp)) {
            return temp;
        } else {
            return loc;
        }
    }

    /*	
    *	Determines the distance of the robot, using its avenue() position, 
    *	from the startAve instance variable, in absolute value terms
    *
    *	@return int absolute distance between robot avenue location and startAve
    *	instance variable
    */  	
    public double getDistance() {
    
        Location loc = location();
        
        int deltaStr = loc.row() - startStr;
        int deltaAve = loc.col() - startAve;
    
        return Math.sqrt(Math.pow(deltaStr, 2) + Math.pow(deltaAve, 2));
    }
    
    /*	
    *	reports the total number of steps (numSteps instance variable) the robot has taken 
    *	since its construction
    *
    *	@return int number of steps taken by the robot
    */  	
    public int getNumSteps() {
    
        return numSteps;
    }
    
}