/** 
 * Walker.java
 *
 * Title:	RandomWalker Demo		
 * Description:	This robot has it's own "coin" to randomly generate its
 *		next move. chooses its next step and moves there if the path is not blocked,
 *		as well as how many steps it took.
 *
 * 
 * @author		Slater		
 * @version			
 */


package kareltherobot;

import java.util.*;


class Walker extends MyRobot 
{

	static private Random coin = new Random();
	private int startAve, startSt, numSteps;
	
        /**
        *	constructor initially sets up using the parent c'tor, then initializes
        *	the Random coin object and state
        */ 
	public Walker(int street, int avenue, Directions.Direction direction, int howmany) 
	{
	
		super(street, avenue, direction, howmany);
		startSt = street;
		startAve = avenue;
		numSteps = 0;
	}

        /**
        * 	The inherited move() method is overwritten, to allow the 
        *	robot to choose its path, as well as modify its position
        *	and update the number of steps. It will also prevent the robot
        *	from running into a wall
        */	
	public void move()
	{
            int flip  = coin.nextInt(2);		// returns possible values 0 or 1
		
            if (flip == 0) {
                faceEast();
            } else {			
                faceWest();
            }
            
            if (frontIsClear()) {
                super.move();
                numSteps++;
            }
        }


        /*	
        *	Determines the distance of the robot, using its avenue() position, 
        *	from the startAve instance variable, in absolute value terms
        *
        *	@return int absolute distance between robot avenue location and startAve
        *	instance variable
        */  	
	public double getDistance() {
	
            return Math.abs(avenue() - startAve);
	}
        
        /*	
        *	reports the total number of steps (numSteps instance variable) the robot has taken 
        *	since its construction
        *
        *	@return int number of steps taken by the robot
        */  	
        public int getNumSteps() {
        
            return numSteps;
        }
	
}